//
//  BNCamera.m
//  Camera
//
//  Created by Ravil on 15.08.13.
//  Copyright (c) 2013 bossnote. All rights reserved.
//

#import "BNCamera.h"

@implementation BNCamera

-(void)createCameraWithType:(BNCameraType)type  toLayer:(CALayer *)parrentLayer{
    switch (type) {
        case simplePhoto:{
            currPreset = AVCaptureSessionPresetPhoto;
            _currentPosition = AVCaptureDevicePositionBack;
        } break;
        case video:{
            currPreset = AVCaptureSessionPresetHigh;
            _currentPosition = AVCaptureDevicePositionBack;
        } break;
        case document:{
            currPreset = AVCaptureSessionPresetPhoto;
        } break;
            
        default:
            break;
    }
    
    _currentPosition = AVCaptureDevicePositionBack;
    mainLayer = parrentLayer;
    changeModQueue = dispatch_queue_create("changeModeAndCreateCameraLayer", NULL);
    
    dispatch_async(changeModQueue, ^{
        [self reinitCamera];
        dispatch_async(dispatch_get_main_queue(), ^{
            [mainLayer addSublayer:captureVideoPreviewLayer];
        });
    });
    
    self.currentType = type;
    
}

-(void)flipCamera {
    
    _currentPosition = _currentPosition == AVCaptureDevicePositionBack?AVCaptureDevicePositionFront:AVCaptureDevicePositionBack;
    
    CATransition *animation = [CATransition animation];
    [animation setDelegate:self];
    [animation setDuration:0.5f];
    CAMediaTimingFunction *tf = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
    [animation setTimingFunction:tf];
    [animation setType:@"alignedFlip"];
    [mainLayer addAnimation:animation forKey:NULL];
    
    dispatch_async(changeModQueue, ^{
        [self stopSession];
        [self reinitCamera];
        dispatch_async(dispatch_get_main_queue(), ^{
            [mainLayer addSublayer:captureVideoPreviewLayer];
        });
    });

}

-(void)setFocusPoint:(CGPoint)point {
    if ([captureDevice isFocusPointOfInterestSupported] && [captureDevice isFocusModeSupported:AVCaptureFocusModeAutoFocus]) {
        NSError *error;
        if ([captureDevice lockForConfiguration:&error]) {
            [captureDevice setFocusPointOfInterest:[self convertToPointOfInterestFromViewCoordinates:point]];
            [captureDevice setFocusMode:AVCaptureFocusModeAutoFocus];
            [captureDevice unlockForConfiguration];
        } else {
            
        }
        
    }
}

- (void)setOrientation:(UIDeviceOrientation)orientation {
    if ([videoConnection isVideoOrientationSupported]) [videoConnection setVideoOrientation:orientation];
}

-(void)zoomCameraWithGesture:(UIPinchGestureRecognizer *)gesture {
    switch (gesture.state) {
        case UIGestureRecognizerStateBegan:
        {
            beginScale = scaleFactor;
        }
            break;
            
        default:
            break;
    }
    
    scaleFactor = beginScale * gesture.scale;
    if (scaleFactor < 1)scaleFactor = 1;
    if (scaleFactor > [videoConnection videoMaxScaleAndCropFactor]) scaleFactor = [videoConnection videoMaxScaleAndCropFactor];
    
    [CATransaction begin];
    [CATransaction setAnimationDuration:.025];
    [captureVideoPreviewLayer setAffineTransform:CGAffineTransformMakeScale(scaleFactor, scaleFactor)];
    [CATransaction commit];
    
    [videoConnection setVideoScaleAndCropFactor:scaleFactor];

}

-(void)setFlashMode:(AVCaptureFlashMode)mode {
    
    if ([captureDevice isFlashAvailable]) {
        [captureDevice lockForConfiguration:nil];
        
        captureDevice.flashMode = mode;
        
        [captureDevice unlockForConfiguration];
    }
}

- (void) stopSession {
    
    
    [session stopRunning];
        
    for (AVCaptureInput *input in session.inputs) {
        [session removeInput:input];
    }
    
    for (AVCaptureOutput *output in session.outputs) {
        [session removeOutput:output];
    }
        
    [captureVideoPreviewLayer removeFromSuperlayer];

}

-(void)capturePhoto {
    
    [stillImageOutput captureStillImageAsynchronouslyFromConnection:videoConnection completionHandler: ^(CMSampleBufferRef imageSampleBuffer, NSError *error)
     {
         CFDictionaryRef exifAttachments = CMGetAttachment( imageSampleBuffer, kCGImagePropertyExifDictionary, NULL);
         if (exifAttachments)
         {
             // Do something with the attachments.
             NSLog(@"attachements: %@", exifAttachments);
         } else {
             NSLog(@"no attachments");
         }
         
         NSData *imageData = [AVCaptureStillImageOutput jpegStillImageNSDataRepresentation:imageSampleBuffer];
         UIImage *image = [[UIImage alloc] initWithData:imageData];
         
         
         UIImageWriteToSavedPhotosAlbum(image, nil, nil, nil);
     }];
}

-(void)stopVideoRecord {
    _isRecordingVideo = NO;
    
    [movieOutput stopRecording];
}

-(void)startVideoRecord {
    _isRecordingVideo = YES;
    
    NSString * filePath = [self createFilePath];
    if ([[NSFileManager defaultManager] fileExistsAtPath:filePath]) {
        [[NSFileManager defaultManager] removeItemAtPath:filePath error:nil];
    }
    
    [videoConnection setVideoOrientation:[UIDevice currentDevice].orientation];

    NSURL * outputUrl = [NSURL fileURLWithPath:filePath];
    [movieOutput startRecordingToOutputFileURL:outputUrl recordingDelegate:self];
}

-(NSString *)createFilePath {
    NSArray *paths                  = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory    = [paths objectAtIndex:0];
    
    return [documentsDirectory stringByAppendingPathComponent:@"video"];
}

- (AVCaptureConnection *)connectionWithMediaType:(NSString *)mediaType fromConnections:(NSArray *)connections
{
	for ( AVCaptureConnection *connection in connections ) {
		for ( AVCaptureInputPort *port in [connection inputPorts] ) {
			if ( [[port mediaType] isEqual:mediaType] ) {
				return connection;
			}
		}
	}
    
	return nil;
}

-(void)captureOutput:(AVCaptureFileOutput *)captureOutput didFinishRecordingToOutputFileAtURL:(NSURL *)outputFileURL fromConnections:(NSArray *)connections error:(NSError *)error {
    
}

-(void)captureOutput:(AVCaptureFileOutput *)captureOutput didStartRecordingToOutputFileAtURL:(NSURL *)fileURL fromConnections:(NSArray *)connections {
    
}

-(void)changeType:(BNCameraType)type {
    BNCameraType lastType = _currentType;
    switch (lastType) {
        case document:
        case simplePhoto: {
            if (type == video || (type == document && _currentPosition == AVCaptureDevicePositionFront)) {
                
                CATransition *animation = [CATransition animation];
                [animation setDelegate:self];
                [animation setDuration:0.5f];
                CAMediaTimingFunction *tf = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
                [animation setTimingFunction:tf];
                [animation setType:@"rippleEffect"];
                [mainLayer addAnimation:animation forKey:NULL];
                
                if (type == document && _currentPosition == AVCaptureDevicePositionFront) _currentPosition = AVCaptureDevicePositionBack;
                if (type == video) currPreset = AVCaptureSessionPresetHigh;
                
                dispatch_async(changeModQueue, ^{
                    [self stopSession];
                    _currentType = type;
                    [self reinitCamera];
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [mainLayer addSublayer:captureVideoPreviewLayer ];
                    });
                });
            }
        }
            break;
        case video: {
            if (type == simplePhoto || type == document) {
                
                CATransition *animation = [CATransition animation];
                [animation setDelegate:self];
                [animation setDuration:0.5f];
                CAMediaTimingFunction *tf = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
                [animation setTimingFunction:tf];
                [animation setType:@"rippleEffect"];
                [mainLayer addAnimation:animation forKey:NULL];
                
                if (type == document && _currentPosition == AVCaptureDevicePositionFront) _currentPosition = AVCaptureDevicePositionBack;
                currPreset = AVCaptureSessionPresetPhoto;
                
                dispatch_async(changeModQueue, ^{
                    [self stopSession];
                    _currentType = type;
                    [self reinitCamera];
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [mainLayer addSublayer:captureVideoPreviewLayer ];
                    });
                });
            }
        }
            break;
            
        default:
            break;
    }
    
    _currentType = type;
}



- (void)reinitCamera{

//    if (!session)
        session = [[AVCaptureSession alloc] init];
    
    [session beginConfiguration];
    
    scaleFactor = 1;
    
    session.sessionPreset = currPreset;
    
    captureVideoPreviewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:session];
    captureVideoPreviewLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
    
    captureVideoPreviewLayer.frame = [UIScreen mainScreen].bounds;
        
    captureDevice = [self cameraInPosition:_currentPosition];
    
    
    
    if ([captureDevice isFlashAvailable]) {
        
        [captureDevice lockForConfiguration:nil];
        
        captureDevice.flashMode = AVCaptureFlashModeAuto;
        
        [captureDevice unlockForConfiguration];
        
    }
    
    
    NSError *error = nil;
    AVCaptureDeviceInput *input = [AVCaptureDeviceInput deviceInputWithDevice:captureDevice error:&error];
    if (!input) {
        NSLog(@"ERROR: trying to open camera: %@", error);
    }
    [session addInput:input];
    
//    if (_currentType == video) {
        AVCaptureDevice * device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeAudio];
        AVCaptureDeviceInput *inputN = [AVCaptureDeviceInput deviceInputWithDevice:device error:&error];
        [session addInput:inputN];
        
        movieOutput = [[AVCaptureMovieFileOutput alloc] init];
        [session addOutput:movieOutput];
        
//    } 
        stillImageOutput = [[AVCaptureStillImageOutput alloc] init];
        NSDictionary *outputSettings = [[NSDictionary alloc] initWithObjectsAndKeys: AVVideoCodecJPEG, AVVideoCodecKey, nil];
        [stillImageOutput setOutputSettings:outputSettings];
        
        [session addOutput:stillImageOutput];
        
    videoConnection = [self connectionWithMediaType:AVMediaTypeVideo fromConnections:_currentType == video? [movieOutput connections] : [stillImageOutput connections]];
    if ([videoConnection isVideoOrientationSupported])
        [videoConnection setVideoOrientation:[UIDevice currentDevice].orientation];
    
    [session commitConfiguration];
    [session startRunning];
}

- (AVCaptureDevice *)cameraInPosition:(AVCaptureDevicePosition)position
{
    NSArray *devices = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
    for (AVCaptureDevice *device in devices) {
        if ([device position] == position) {
            return device;
        }
    }
    
    return nil;
}

- (CGPoint)convertToPointOfInterestFromViewCoordinates:(CGPoint)viewCoordinates
{
    CGPoint pointOfInterest = CGPointMake(.5f, .5f);
    CGSize frameSize = [[UIScreen mainScreen] bounds].size;
    
    if ([videoConnection isVideoMirrored]) {
        viewCoordinates.x = frameSize.width - viewCoordinates.x;
    }
    
    if ( [[captureVideoPreviewLayer videoGravity] isEqualToString:AVLayerVideoGravityResize] ) {
		// Scale, switch x and y, and reverse x
        pointOfInterest = CGPointMake(viewCoordinates.y / frameSize.height, 1.f - (viewCoordinates.x / frameSize.width));
    } else {
        CGRect cleanAperture;
        for (AVCaptureInputPort *port in [videoConnection inputPorts]) {
            if ([port mediaType] == AVMediaTypeVideo) {
                cleanAperture = CMVideoFormatDescriptionGetCleanAperture([port formatDescription], YES);
                CGSize apertureSize = cleanAperture.size;
                CGPoint point = viewCoordinates;
                
                CGFloat apertureRatio = apertureSize.height / apertureSize.width;
                CGFloat viewRatio = frameSize.width / frameSize.height;
                CGFloat xc = .5f;
                CGFloat yc = .5f;
                
                if ( [[captureVideoPreviewLayer videoGravity] isEqualToString:AVLayerVideoGravityResizeAspect] ) {
                    if (viewRatio > apertureRatio) {
                        CGFloat y2 = frameSize.height;
                        CGFloat x2 = frameSize.height * apertureRatio;
                        CGFloat x1 = frameSize.width;
                        CGFloat blackBar = (x1 - x2) / 2;
						// If point is inside letterboxed area, do coordinate conversion; otherwise, don't change the default value returned (.5,.5)
                        if (point.x >= blackBar && point.x <= blackBar + x2) {
							// Scale (accounting for the letterboxing on the left and right of the video preview), switch x and y, and reverse x
                            xc = point.y / y2;
                            yc = 1.f - ((point.x - blackBar) / x2);
                        }
                    } else {
                        CGFloat y2 = frameSize.width / apertureRatio;
                        CGFloat y1 = frameSize.height;
                        CGFloat x2 = frameSize.width;
                        CGFloat blackBar = (y1 - y2) / 2;
						// If point is inside letterboxed area, do coordinate conversion. Otherwise, don't change the default value returned (.5,.5)
                        if (point.y >= blackBar && point.y <= blackBar + y2) {
							// Scale (accounting for the letterboxing on the top and bottom of the video preview), switch x and y, and reverse x
                            xc = ((point.y - blackBar) / y2);
                            yc = 1.f - (point.x / x2);
                        }
                    }
                } else if ([[captureVideoPreviewLayer videoGravity] isEqualToString:AVLayerVideoGravityResizeAspectFill]) {
					// Scale, switch x and y, and reverse x
                    if (viewRatio > apertureRatio) {
                        CGFloat y2 = apertureSize.width * (frameSize.width / apertureSize.height);
                        xc = (point.y + ((y2 - frameSize.height) / 2.f)) / y2; // Account for cropped height
                        yc = (frameSize.width - point.x) / frameSize.width;
                    } else {
                        CGFloat x2 = apertureSize.height * (frameSize.height / apertureSize.width);
                        yc = 1.f - ((point.x + ((x2 - frameSize.width) / 2)) / x2); // Account for cropped width
                        xc = point.y / frameSize.height;
                    }
                }
                
                pointOfInterest = CGPointMake(xc, yc);
                break;
            }
        }
    }
    
    return pointOfInterest;
}


@end
