//
//  BNTransitions.m
//  transition
//
//  Created by Ravil on 05.08.13.
//  Copyright (c) 2013 bossnote. All rights reserved.
//

#import "BNTransitions.h"

@implementation BNScaleTransition

+(BNScaleTransition *)initwith:(UIView *)destView source:(UIView *)sourceView main:(UIView *)mainView width:(CGFloat)destWidth{
    BNScaleTransition * selfObjecrt = [[BNScaleTransition alloc] init];
    selfObjecrt.sourceView = sourceView;
    selfObjecrt.destView = destView;
    selfObjecrt.mainView = mainView;
    
    selfObjecrt.destWid = destWidth?destWidth:mainView.frame.size.width;
        
    [mainView addSubview:destView];
    
    selfObjecrt->viewPosition = [destView makePositionWithConstraints:selfObjecrt.destWid];
    [sourceView makeSimpleCenter];
    
    return selfObjecrt;
}

-(void)setXpos:(CGFloat)xpos {
    _xpos = xpos;
    
    viewPosition.constant = xpos;
    
    CGFloat  scalefactor = xpos<0? (1-((xpos+_destWid)/(_destWid*4))):(1-((_destWid-xpos)/(_destWid*4)));
    if (scalefactor>1) scalefactor = 1;
    _sourceView.transform = CGAffineTransformMakeScale(scalefactor, scalefactor);
}

- (void)done {
	[UIView animateWithDuration:.3 animations:^{
        _sourceView.transform = CGAffineTransformMakeScale(0.7, 0.7);
        viewPosition.constant = 0;
        [_mainView layoutIfNeeded];

	} completion:^(BOOL finished) {
        [_sourceView removeFromSuperview];
        _sourceView.transform = CGAffineTransformMakeScale(1, 1);
        [_destView makeFlexible];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"TransitionDone" object:nil];
        
	}];
}

-(void)cancel {
    [UIView animateWithDuration:.3 animations:^{
        _sourceView.transform = CGAffineTransformMakeScale(1, 1);
		viewPosition.constant = _xpos>0?_destWid:-_destWid;
        [_mainView layoutIfNeeded];
	} completion:^(BOOL finished) {
        [_destView removeFromSuperview];
        if (_sourceView) [_sourceView makeFlexible];
         [[NSNotificationCenter defaultCenter] postNotificationName:@"TransitionDone" object:nil];
	}];
}

@end


@implementation UIView (ConstraintsAddition)

-(NSArray *)makePositionByOldFrame:(CGPoint)startPoint {
//    [self.superview removeAllConstraints];
    
    [self remoOnlyNededConstraints];
    
    NSLayoutConstraint * widthConst = [NSLayoutConstraint constraintWithItem:self
                                                                   attribute:NSLayoutAttributeWidth
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:nil
                                                                   attribute:NSLayoutAttributeNotAnAttribute
                                                                  multiplier:1.0
                                                                    constant:self.frame.size.width];
    
    NSLayoutConstraint * leftConst = [NSLayoutConstraint constraintWithItem:self
                                                                  attribute:NSLayoutAttributeLeading
                                                                  relatedBy:NSLayoutRelationEqual
                                                                     toItem:self.superview
                                                                  attribute:NSLayoutAttributeLeading
                                                                 multiplier:1.0
                                                                   constant:startPoint.x];
    
    NSLayoutConstraint * topConst = [NSLayoutConstraint constraintWithItem:self
                                                                  attribute:NSLayoutAttributeTop
                                                                  relatedBy:NSLayoutRelationEqual
                                                                     toItem:self.superview
                                                                  attribute:NSLayoutAttributeTop
                                                                 multiplier:1.0
                                                                   constant:startPoint.y];
    
    NSLayoutConstraint * heightConst = [NSLayoutConstraint constraintWithItem:self
                                                                 attribute:NSLayoutAttributeHeight
                                                                 relatedBy:NSLayoutRelationEqual
                                                                    toItem:nil
                                                                 attribute:NSLayoutAttributeNotAnAttribute
                                                                multiplier:1.0
                                                                  constant:self.frame.size.height];
    NSArray * some = @[leftConst,topConst,widthConst,heightConst];
    
    [self.superview addConstraints:some];
    [self.superview layoutIfNeeded];
    
    return some;
}

-(NSArray *)makeBottomPositionByOldFrame:(CGPoint)startPoint {
    //    [self.superview removeAllConstraints];
    
    [self remoOnlyNededConstraints];
    
    NSLayoutConstraint * widthConst = [NSLayoutConstraint constraintWithItem:self
                                                                   attribute:NSLayoutAttributeWidth
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:nil
                                                                   attribute:NSLayoutAttributeNotAnAttribute
                                                                  multiplier:1.0
                                                                    constant:self.frame.size.width];
    
    NSLayoutConstraint * leftConst = [NSLayoutConstraint constraintWithItem:self
                                                                  attribute:NSLayoutAttributeLeading
                                                                  relatedBy:NSLayoutRelationEqual
                                                                     toItem:self.superview
                                                                  attribute:NSLayoutAttributeLeading
                                                                 multiplier:1.0
                                                                   constant:startPoint.x];
    
    NSLayoutConstraint * bottomConst = [NSLayoutConstraint constraintWithItem:self
                                                                 attribute:NSLayoutAttributeBottom
                                                                 relatedBy:NSLayoutRelationEqual
                                                                    toItem:self.superview
                                                                 attribute:NSLayoutAttributeBottom
                                                                multiplier:1.0
                                                                  constant:self.superview.frame.size.height - (startPoint.y+self.frame.size.height)];
    
    NSLayoutConstraint * heightConst = [NSLayoutConstraint constraintWithItem:self
                                                                    attribute:NSLayoutAttributeHeight
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:nil
                                                                    attribute:NSLayoutAttributeNotAnAttribute
                                                                   multiplier:1.0
                                                                     constant:self.frame.size.height];
    NSArray * some = @[leftConst,bottomConst,widthConst,heightConst];
    
    [self.superview addConstraints:some];
    [self.superview layoutIfNeeded];
    
    return some;
}

-(NSLayoutConstraint *)makePositionWithConstraints:(CGFloat)width{
    [self.superview removeAllConstraints];
    
//    [self remoOnlyNededConstraints];
    
    NSMutableArray * constraints = [NSMutableArray array];
    NSDictionary * views = @{@"destView":self};
    [constraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|[destView]|"
                                                                             options:0
                                                                             metrics:nil
                                                                               views:views]];
    NSLayoutConstraint * widthConst = [NSLayoutConstraint constraintWithItem:self
                                                                   attribute:NSLayoutAttributeWidth
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:nil
                                                                   attribute:NSLayoutAttributeNotAnAttribute
                                                                  multiplier:1.0
                                                                    constant:width];
    [constraints addObject:widthConst];
    
    NSLayoutConstraint * leftConst = [NSLayoutConstraint constraintWithItem:self
                                                                   attribute:NSLayoutAttributeLeading
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:self.superview
                                                                   attribute:NSLayoutAttributeLeading
                                                                  multiplier:1.0
                                                                    constant:-width];
    [constraints addObject:leftConst];
    
    [self.superview addConstraints:constraints];
    
    return leftConst;
}

-(NSLayoutConstraint *)makeSimpleCenter {
    
    NSLayoutConstraint * constraintX = [NSLayoutConstraint
                                        constraintWithItem:self
                                        attribute:NSLayoutAttributeCenterX
                                        relatedBy:NSLayoutRelationEqual
                                        toItem:self.superview
                                        attribute:NSLayoutAttributeCenterX
                                        multiplier:1.0f
                                        constant:0];
    
    NSLayoutConstraint * constraintY = [NSLayoutConstraint
                                        constraintWithItem:self
                                        attribute:NSLayoutAttributeCenterY
                                        relatedBy:NSLayoutRelationEqual
                                        toItem:self.superview
                                        attribute:NSLayoutAttributeCenterY
                                        multiplier:1.0f
                                        constant:0];
    NSLayoutConstraint * widthConst = [NSLayoutConstraint constraintWithItem:self
                                                                   attribute:NSLayoutAttributeWidth
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:nil
                                                                   attribute:NSLayoutAttributeNotAnAttribute
                                                                  multiplier:1.0
                                                                    constant:self.frame.size.width];
    
    NSLayoutConstraint * heightConst = [NSLayoutConstraint constraintWithItem:self
                                                                    attribute:NSLayoutAttributeHeight
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:nil
                                                                    attribute:NSLayoutAttributeNotAnAttribute
                                                                   multiplier:1.0
                                                                     constant:self.frame.size.height];
    
    
    [self.superview addConstraints:@[constraintX,constraintY,heightConst,widthConst]];
    
    return constraintY;
}


-(NSLayoutConstraint *)makeCenterAllign:(CGFloat)xConstr {
    
    NSLayoutConstraint * constraintX = [NSLayoutConstraint
                                       constraintWithItem:self
                                       attribute:NSLayoutAttributeCenterX
                                       relatedBy:NSLayoutRelationEqual
                                       toItem:self.superview
                                       attribute:NSLayoutAttributeCenterX
                                       multiplier:1.0f
                                       constant:0];
    
    NSLayoutConstraint * constraintY = [NSLayoutConstraint
                                        constraintWithItem:self
                                        attribute:NSLayoutAttributeCenterY
                                        relatedBy:NSLayoutRelationEqual
                                        toItem:self.superview
                                        attribute:NSLayoutAttributeCenterY
                                        multiplier:1.0f
                                        constant:xConstr];
    

    
    [self.superview addConstraints:@[constraintX,constraintY]];
    
    return constraintY;
}

-(void)makePositionFromRect:(CGRect)frame {
    
    NSLayoutConstraint * widthConst = [NSLayoutConstraint constraintWithItem:self
                                                                   attribute:NSLayoutAttributeWidth
                                                                   relatedBy:NSLayoutRelationEqual
                                                                      toItem:nil
                                                                   attribute:NSLayoutAttributeNotAnAttribute
                                                                  multiplier:1.0
                                                                    constant:frame.size.width];
    
    NSLayoutConstraint * leftConst = [NSLayoutConstraint constraintWithItem:self
                                                                  attribute:NSLayoutAttributeLeading
                                                                  relatedBy:NSLayoutRelationEqual
                                                                     toItem:self.superview
                                                                  attribute:NSLayoutAttributeLeading
                                                                 multiplier:1.0
                                                                   constant:frame.origin.x];
    
    NSLayoutConstraint * topConst = [NSLayoutConstraint constraintWithItem:self
                                                                 attribute:NSLayoutAttributeTop
                                                                 relatedBy:NSLayoutRelationEqual
                                                                    toItem:self.superview
                                                                 attribute:NSLayoutAttributeTop
                                                                multiplier:1.0
                                                                  constant:frame.origin.y];
    
    NSLayoutConstraint * heightConst = [NSLayoutConstraint constraintWithItem:self
                                                                    attribute:NSLayoutAttributeHeight
                                                                    relatedBy:NSLayoutRelationEqual
                                                                       toItem:nil
                                                                    attribute:NSLayoutAttributeNotAnAttribute
                                                                   multiplier:1.0
                                                                     constant:frame.size.height];
//    NSArray * some = @[leftConst,topConst,widthConst,heightConst];
    
    [self.superview addConstraints:@[leftConst,topConst]];
    [self addConstraints:@[widthConst,heightConst]];
    [self.superview layoutIfNeeded];
}

-(void)makeFlexible {
    [self.superview removeAllConstraints];
    
//    [self remoOnlyNededConstraints];
    
    NSArray *arr;
    
    NSDictionary * views = @{@"destView":self};
    
    //horizontal constraints
    arr = [NSLayoutConstraint constraintsWithVisualFormat:@"H:|[destView]|"
                                                  options:0
                                                  metrics:nil
                                                    views:views];
    [self.superview addConstraints:arr];
    
    //vertical constraints
    arr = [NSLayoutConstraint constraintsWithVisualFormat:@"V:|[destView]|"
                                                  options:0
                                                  metrics:nil
                                                    views:views];
    [self.superview addConstraints:arr];
}

-(void)makeFlexibleWeak {
//    [self.superview removeAllConstraints];
    
    [self remoOnlyNededConstraints];
    
    NSArray *arr;
    
    NSDictionary * views = @{@"destView":self};
    
    //horizontal constraints
    arr = [NSLayoutConstraint constraintsWithVisualFormat:@"H:|[destView]|"
                                                  options:0
                                                  metrics:nil
                                                    views:views];
    [self.superview addConstraints:arr];
    
    //vertical constraints
    arr = [NSLayoutConstraint constraintsWithVisualFormat:@"V:|[destView]|"
                                                  options:0
                                                  metrics:nil
                                                    views:views];
    [self.superview addConstraints:arr];
}

-(void)removeAllConstraints {
    [self removeConstraints:self.constraints];
}

-(void)remoOnlyNededConstraints {
    NSMutableArray * mArr = [NSMutableArray array];
    for (NSLayoutConstraint * constraint in self.superview.constraints) {
        if ([constraint.firstItem isEqual:self] || [constraint.secondItem isEqual:self]) {
            [mArr addObject:constraint];
        }
    }
    if (mArr.count>0) [self.superview removeConstraints:mArr];
}

@end
