//
//  BNViewController.m
//  Camera
//
//  Created by Ravil on 13.08.13.
//  Copyright (c) 2013 bossnote. All rights reserved.
//

#import "BNViewController.h"

@interface BNViewController ()

@end

@implementation BNViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleRotate:)
                                                 name:UIDeviceOrientationDidChangeNotification
                                               object:nil];
    
    UITapGestureRecognizer * tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(focusSelector:)];
    [self.camView addGestureRecognizer:tapGesture];
    
    [self.simplePhotoControls addGestureRecognizer:tapGesture ];
    
    UIPinchGestureRecognizer * zoomGesture = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(zoomSelector:)];
    [self.camView addGestureRecognizer:zoomGesture];
    
    [self.simplePhotoControls addGestureRecognizer:zoomGesture ];
    
    self.documentControl.alpha = 0;
}

- (void)zoomSelector:(UIPinchGestureRecognizer*)gesture {
    [camera zoomCameraWithGesture:gesture];
}

- (void)focusSelector:(UITapGestureRecognizer*)gesture {
    
//    kCATransitionFade
//    kCATransitionMoveIn
//    kCATransitionPush
//    kCATransitionReveal
//    @"cameraIris"
//    @"cameraIrisHollowOpen"
//    @"cameraIrisHollowClose"
//    @"cube"
//    @"alignedCube"
//    @"flip"
//    @"alignedFlip"
//    @"oglFlip"
//    @"rotate"
//    @"pageCurl"
//    @"pageUnCurl"
//    @"rippleEffect"
//    @"suckEffect"
    
    [camera setFocusPoint:[gesture locationInView:_camView]];

}


-(void)handleRotate:(NSNotification*)notification {
    UIDeviceOrientation  orientation = [[UIDevice currentDevice] orientation];
    
    [camera setOrientation:orientation];
    
    CGAffineTransform transform;
    
    switch (orientation) {
        case UIDeviceOrientationLandscapeLeft:
            transform = CGAffineTransformMakeRotation(M_PI_2); // 270 degress
            
            break;
        case UIDeviceOrientationLandscapeRight:
            transform = CGAffineTransformMakeRotation(M_PI+ M_PI_2); // 90 degrees
            break;
        case UIDeviceOrientationPortraitUpsideDown:
            transform= CGAffineTransformMakeRotation(M_PI); // 180 degrees
            break;
        default:
            transform = CGAffineTransformMakeRotation(0.0);

            break;
    }
    
    [UIView animateWithDuration:0.3
                     animations:^{
                         self.closeBTN.transform = transform;
                         _videoBtn.transform = transform;
                         _photoBtn.transform = transform;
                         _docBtn.transform = transform;
                         _flash.transform = transform;
                         _sourceDeviceBtn.transform = transform;
                     }];

    
}

-(void)viewDidAppear:(BOOL)animated {
    
    NSArray * constaraints = [_photoBtn makePositionByOldFrame:_photoBtn.frame.origin];
    
    topLastBtn = constaraints[1];
    leftLastBtn = constaraints[0];
    
    [[[_videoBtn makePositionByOldFrame:_videoBtn.frame.origin] objectAtIndex:1] setConstant:30];
    [[[_docBtn makePositionByOldFrame:_docBtn.frame.origin] objectAtIndex:1] setConstant:30];
        
    topLastBtn.constant = 10;
    
    [_pickerScrol layoutIfNeeded];
    
    _photoBtn.tag = 1;

    
    camera = [BNCamera new];
    
    [camera createCameraWithType:simplePhoto toLayer:_camView.layer];
    
    [self handleRotate:nil];
    
}

- (IBAction)onOffFlash:(UISegmentedControl*)sender {

    AVCaptureFlashMode flashMode;
    
    switch (sender.selectedSegmentIndex) {
        case 0:
            flashMode = AVCaptureFlashModeAuto;
            break;
        case 1:
            flashMode = AVCaptureFlashModeOn;
            break;
        case 2:
            flashMode = AVCaptureFlashModeOff;
            break;
            
        default:
            flashMode = AVCaptureFlashModeAuto;
            break;
    }
    
    [camera setFlashMode:flashMode];
    
}
- (IBAction)changeCamera:(UIButton *)sender {
    
    [camera flipCamera];
    [UIView animateWithDuration:.3
                     animations:^{
                         if (camera.currentPosition == AVCaptureDevicePositionFront) {
                             [_sourceDeviceBtn setTitle:@"Back" forState:UIControlStateNormal];
                             self.flash.alpha = 0;
                         } else {
                             [_sourceDeviceBtn setTitle:@"Front" forState:UIControlStateNormal];
                             self.flash.alpha = 1;
                         }
                     }];
}

-(BOOL)shouldAutorotate {
    return NO;
}

-(NSUInteger)supportedInterfaceOrientations {
    return  UIInterfaceOrientationMaskPortrait;
}

- (IBAction)capture:(UIButton*)sender {
    
    if (isTransition) {
        return;
    }
    
    if (sender.tag == 0) {
        sender.tag = 1;
        [topLastBtn.firstItem setTag:0];
        
        
        
        CGPoint buferPoint = sender.frame.origin;
        
        NSArray * constraints = [sender makePositionByOldFrame:sender.frame.origin];
        NSLayoutConstraint * leftCon = constraints[0];
        NSLayoutConstraint * topCon = constraints[1];
        
        isTransition = YES;
        
        [UIView animateWithDuration:.3
                         animations:^{
                             leftCon.constant = _pickerScrol.frame.size.width/2 - sender.frame.size.width/2;
                             topCon.constant = 10;
                             
                             topLastBtn.constant = buferPoint.y;
                             leftLastBtn.constant = buferPoint.x;
                             
                             if ([sender isEqual:_photoBtn] || [sender isEqual:_videoBtn]) {
                                 _simplePhotoControls.alpha = 1;
                                 _documentControl.alpha = 0;
                             } else {
                                 _simplePhotoControls.alpha = 0;
                                 _documentControl.alpha = 1;
                             }
                             
                             [_pickerScrol layoutIfNeeded];
                             
                         } completion:^(BOOL finished) {
                             topLastBtn = topCon;
                             leftLastBtn = leftCon;
                             isTransition = NO;

                         }];
        BNCameraType newType;
        
        if ([sender isEqual:_photoBtn]) {
            newType = simplePhoto;
        } else if ([sender isEqual:_docBtn]) {
            newType = document;
        } else {
            newType = video;
        }
        
        [camera changeType:newType];
        
    } else {
        
        switch (camera.currentType) {
            case simplePhoto: {
                CATransition *animation = [CATransition animation];
                [animation setDelegate:self];
                [animation setDuration:0.5f];
                CAMediaTimingFunction *tf = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
                [animation setTimingFunction:tf];
                [animation setType:@"cameraIris"];
                [self.view.layer addAnimation:animation forKey:NULL];
                
                [camera capturePhoto];
            }
                break;
            case video: {
                CGFloat alpha;
                if (camera.isRecordingVideo) {
                    [camera stopVideoRecord];
                    [_photoBtn setEnabled:YES];
                    [_docBtn setEnabled:YES];
                    alpha = 1;
                } else {
                    [camera startVideoRecord];
                    [_photoBtn setEnabled:NO];
                    [_docBtn setEnabled:NO];
                    alpha = 0;
                }
                [UIView animateWithDuration:.3
                                 animations:^{
                                     _docBtn.alpha = alpha;
                                     _photoBtn.alpha = alpha;
                                 }];
            }
                break;
            default:
                break;
        }
        
        

    }
    
        
    
}
- (IBAction)close:(id)sender {
    [[UIApplication sharedApplication] setStatusBarHidden:NO withAnimation:UIStatusBarAnimationSlide];
    [self dismissViewControllerAnimated:YES completion:nil];
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
